/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.field.shape;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import net.miginfocom.swing.MigLayout;
import org.openide.util.NbBundle;

/**
 * Manage stored shape property mappings.
 * 
 * @author johan
 */
@NbBundle.Messages({
        "ShapePropertyMappingsPanel delete=Delete selected mappings",
        "ShapePropertyMappingsPanel list title=Registered mappings"
})
public class ShapePropertyMappingsPanel extends JPanel {
    private static final int MAPPINGS_LIST_WIDTH = 200;
    private static final int MAPPINGS_LIST_HEIGHT = 300;

    private final ShapePropertyMappingRegistry registry = ShapePropertyMappingRegistry.getInstance();
    private final RegistryListener registryListener = new RegistryListener();
    private final JList<ShapePropertyMapping> mappingsList;
    private final JButton deleteButton;

    public ShapePropertyMappingsPanel() {
        setLayout(new MigLayout("wrap 2, insets 0 8 0 0"));
        
        mappingsList = new JList<>();
        mappingsList.setPreferredSize(new Dimension(MAPPINGS_LIST_WIDTH, MAPPINGS_LIST_HEIGHT));
        mappingsList.setBorder(new TitledBorder(Bundle.ShapePropertyMappingsPanel_list_title()));
                
        deleteButton = new JButton(Bundle.ShapePropertyMappingsPanel_delete());
        deleteButton.setEnabled(!mappingsList.isSelectionEmpty());
        deleteButton.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                registry.removeAll(mappingsList.getSelectedValuesList());
            }
        });
        
        mappingsList.addListSelectionListener(new ListSelectionListener() {

            @Override
            public void valueChanged(ListSelectionEvent e) {
                deleteButton.setEnabled(!mappingsList.isSelectionEmpty());
            }
        });
        
        // TODO (feature, maybe later): reorder the list (move elements up/down, or to first/last)
        
        add(mappingsList);
        add(deleteButton);
        
        registry.addPropertyChangeListener(ShapePropertyMappingRegistry.PROP_MAPPINGS, registryListener);
        registryListener.propertyChange(null);
    }
    
    private class RegistryListener implements PropertyChangeListener {

        @Override
        public void propertyChange(PropertyChangeEvent evt) {
            mappingsList.setListData(registry.findAll().toArray(new ShapePropertyMapping[]{}));
        }
    }
}
